from typing import Any, Optional
from xml.sax.xmlreader import XMLReader

def parse(file: str, parser: Optional[XMLReader] = ..., bufsize: Optional[int] = ...): ...
def parseString(string: str, parser: Optional[XMLReader] = ...): ...
def __getattr__(name: str) -> Any: ...  # incomplete
